/*	$Id: spaceinvaders.h,v 1.31 2017/05/24 17:59:20 hacki Exp $ */

/*
 * Copyright (c) 2017 Marcus Glocker <marcus@nazgul.ch>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Macros.
 */
#define debug_write(a, ...)	DM(a, ...)

/*
 * System.
 */
#define VERSION			"0.6"
#define FILENAME_HISCORE	".spaceinvaders.scores"
#define FILENAME_DEBUG		"spaceinvaders.log"
volatile sig_atomic_t		lock_signal = 0;
int				debug_fd = 0;
int				silent_mode;
extern char			*__progname;

/*
 * Enumerations.
 */
enum {
	MODE_USAGE_HELP,
	MODE_USAGE_VERSION,
	MODE_CTRL_INIT_FULL,
	MODE_CTRL_INIT_PARTIAL,
	MODE_BOARD_INIT_FULL,
	MODE_BOARD_INIT_INTRO
};

enum {
	BBS_BACKUP,
	BBS_RESTORE,
	BBS_REMOVE_ALIENS,
	BBS_REMOVE_ALIENS_MISSILE
};

enum {
	DIRECTION_RIGHT,
	DIRECTION_LEFT,
	DIRECTION_DOWN
};

/*
 * Sprites.
 */
#define SPRITES_NUM 15

enum {
	SPACE,
	LASER_BASE,
	LASER_SHOT,
	SHIELD_3,
	SHIELD_2,
	SHIELD_1,
	ALIEN_1,
	ALIEN_2,
	ALIEN_3,
	ALIEN_1_MISSILE,
	ALIEN_2_MISSILE,
	ALIEN_3_MISSILE,
	MYSTERY_SHIP,
	FLOOR,
	EXPLOSION
};

static const struct sprite {
	char	graph;
	int	score;
	int	damage;
	int	protection;
} sprites[SPRITES_NUM] = {
	{ ' ',  0, 0, 0 },	/* Space */
	{ 'M',  0, 0, 1 },	/* Laser base */
	{ '|',  0, 1, 0 },	/* Laser shot */
	{ 'X',  0, 0, 3 },	/* Shield 1 */
	{ 'x',  0, 0, 2 },	/* Shield 2 */
	{ '_',  0, 1, 1 },	/* Shield 3 */
	{ 'Y', 30, 0, 1 },	/* Alien 1 */
	{ 'O', 20, 0, 1 },	/* Alien 2 */
	{ 'H', 10, 0, 1 },	/* Alien 3 */
	{ '#',  0, 3, 1 },	/* Alien 1 missile */
	{ '*',  0, 2, 1 },	/* Alien 2 missile */
	{ '.',  0, 1, 1 },	/* Alien 3 missile */
	{ 'T', 50, 0, 1 },	/* Mystery ship */
	{ '-',  0, 0, 0 },	/* Floor */
	{ '@',  0, 0, 0 }	/* Explosion */
};

/*
 * Game board and control.
 */
#define SCREENS			10
#define BOARD_ROWS		24
#define BOARD_COLS		80
#define BOARD_UPDATE_USEC	10000
#define ALIENS_MOVE_USEC	1000000
#define ALIENS_MOVE_MIN_USEC	100000
#define ALIENS_SHOT_USEC	1000000
#define ALIENS_MISSILE_USEC	50000
#define LASER_SHOT_USEC		30000
#define MYSTERY_TRIGGER_USEC	1000000
#define MYSTERY_MOVE_USEC	200000
#define MYSTERY_LASER_TOP	23
#define TEXT_CLEANUP_USEC	4000000
#define PAUSE_SEC		3
#define PLAYER_LIVES		3
#define ALIENS_SHOT_MAX		3
#define ALIENS_PER_LINE		11
#define ALIENS_NUM		55
#define SCORE_MAX		9990
#define SCORE_EXTRA_LIVE	1500

char board[BOARD_ROWS][BOARD_COLS];
char board_bs[BOARD_ROWS][BOARD_COLS];	/* Board Backing Store */

struct control {
	int	screen;
	int	player_score;
	int	player_hiscore;
	int	player_hiscore_new;
	int	player_lives;
	int	player_pos_x;
	int	player_shot_inprogress;
	int	player_laser_cycle;
	int	player_laser_counter;
	int	aliens_num;
	int	aliens_shot_inprogress;
	int	aliens_dir_now;
	int	aliens_dir_last;
	int	aliens_move_cycle;
	int	aliens_move_usec;
	int	aliens_missile_trigger_cycle;
	int	aliens_missile_cycle;
	int	mystery_trigger_cycle;
	int	mystery_move_cycle;
	int	mystery_inprogress;
	int	mystery_laser_top;
	int	mystery_dir;
	int	msg_cleanup_cycle;
	int	score_extra_live;
	int	game_over;
	int	game_pause;
} ctrl;

/*
 * Texts.
 */
char *txt_score = "SCORE";
char *txt_hiscore = "HI-SCORE";
char *msg_hiscore_new = "NEW HI-SCORE REACHED";
char *msg_mystery = "MYSTERY SCORE";
char *msg_extra_live = "EXTRA LIVE RECEIVED";
char *msg_player_wins = "YOU WIN";
char *msg_game_pause = "PAUSE [P = RESUME]";
char *msg_game_over = "GAME OVER [S = START NEW GAME, Q = QUIT]";
